"""Point mass element module for STOCHASTIC ROSS.

This module creates an instance of random point mass for stochastic
analysis.
"""

from ross.point_mass import PointMass
from ross.stochastic.st_results_elements import plot_histogram
from ross.units import check_units

__all__ = ["ST_PointMass", "st_pointmass_example"]


class ST_PointMass:
    """Random point mass element.

    Creates an object containing a list with random instances of PointMass.

    Parameters
    ----------
    n: int
        Node in which the disk will be inserted.
    m: float, list, optional
        Mass for the element.
        Input a list to make it random.
    mx: float, list optional
        Mass for the element on the x direction.
        Input a list to make it random.
    my: float, optional
        Mass for the element on the y direction.
        Input a list to make it random.
    tag : str, optional
        A tag to name the element
        Default is None
    color : str, optional
        A color to be used when the element is represented.
        Default is "DarkSalmon".
    is_random : list
        List of the object attributes to become random.
        Possibilities:
            ["m", "mx", "my"]

    Attributes
    ----------
    elements : list
        display the list with random point mass elements.

    Example
    -------
    >>> import numpy as np
    >>> import ross.stochastic as srs
    >>> elms = srs.ST_PointMass(n=1,
    ...                         mx=np.random.uniform(2.0, 2.5, 5),
    ...                         my=np.random.uniform(2.0, 2.5, 5),
    ...                         is_random=["mx", "my"],
    ...                         )
    >>> len(list(iter(elms)))
    5
    """

    @check_units
    def __init__(
        self,
        n,
        m=None,
        mx=None,
        my=None,
        tag=None,
        color="DarkSalmon",
        is_random=None,
    ):
        attribute_dict = dict(
            n=n,
            m=m,
            mx=mx,
            my=my,
            tag=tag,
            color=color,
        )

        self.is_random = is_random
        self.attribute_dict = attribute_dict

    def __iter__(self):
        """Return an iterator for the container.

        Returns
        -------
        An iterator over random point mass elements.

        Examples
        --------
        >>> import ross.stochastic as srs
        >>> elm = srs.st_pointmass_example()
        >>> len(list(iter(elm)))
        2
        """
        return iter(self.random_var(self.is_random, self.attribute_dict))

    def __getitem__(self, key):
        """Return the value for a given key from attribute_dict.

        Parameters
        ----------
        key : str
            A class parameter as string.

        Raises
        ------
        KeyError
            Raises an error if the parameter doesn't belong to the class.

        Returns
        -------
        Return the value for the given key.

        Example
        -------
        >>> import numpy as np
        >>> import ross.stochastic as srs
        >>> elms = srs.ST_PointMass(n=1,
        ...                         mx=np.random.uniform(2.0, 2.5, 5),
        ...                         my=np.random.uniform(2.0, 2.5, 5),
        ...                         is_random=["mx", "my"],
        ...                         )
        >>> elms["n"]
        1
        """
        if key not in self.attribute_dict.keys():
            raise KeyError("Object does not have parameter: {}.".format(key))

        return self.attribute_dict[key]

    def __setitem__(self, key, value):
        """Set new parameter values for the object.

        Function to change a parameter value.
        It's not allowed to add new parameters to the object.

        Parameters
        ----------
        key : str
            A class parameter as string.
        value : The corresponding value for the attrbiute_dict's key.
            ***check the correct type for each key in ST_PointMass
            docstring.

        Raises
        ------
        KeyError
            Raises an error if the parameter doesn't belong to the class.

        Example
        -------
        >>> import numpy as np
        >>> import ross.stochastic as srs
        >>> elms = srs.ST_PointMass(n=1,
        ...                         mx=np.random.uniform(2.0, 2.5, 5),
        ...                         my=np.random.uniform(2.0, 2.5, 5),
        ...                         is_random=["mx", "my"],
        ...                         )
        >>> elms["mx"] = np.linspace(1.0, 2.0, 5)
        >>> elms["mx"]
        array([1.  , 1.25, 1.5 , 1.75, 2.  ])
        """
        if key not in self.attribute_dict.keys():
            raise KeyError("Object does not have parameter: {}.".format(key))
        self.attribute_dict[key] = value

    def random_var(self, is_random, *args):
        """Generate a list of objects as random attributes.

        This function creates a list of objects with random values for selected
        attributes from ross.PointMass.

        Parameters
        ----------
        is_random : list
            List of the object attributes to become stochastic.
        *args : dict
            Dictionary instanciating the ross.PointMass class.
            The attributes that are supposed to be stochastic should be
            set as lists of random variables.

        Returns
        -------
        f_list : generator
            Generator of random objects.
        """
        args_dict = args[0]
        new_args = []
        for i in range(len(args_dict[is_random[0]])):
            arg = []
            for key, value in args_dict.items():
                if key in is_random:
                    arg.append(value[i])
                else:
                    arg.append(value)
            new_args.append(arg)
        f_list = (PointMass(*arg) for arg in new_args)

        return f_list

    def plot_random_var(self, var_list=None, histogram_kwargs=None, plot_kwargs=None):
        """Plot histogram and the PDF.

        This function creates a histogram to display the random variable
        distribution.

        Parameters
        ----------
        var_list : list, optional
            List of random variables, in string format, to plot.
            Default is plotting all the random variables.
        histogram_kwargs : dict, optional
            Additional key word arguments can be passed to change
            the plotly.go.histogram (e.g. histnorm="probability density", nbinsx=20...).
            *See Plotly API to more information.
        plot_kwargs : dict, optional
            Additional key word arguments can be passed to change the plotly go.figure
            (e.g. line=dict(width=4.0, color="royalblue"), opacity=1.0, ...).
            *See Plotly API to more information.

        Returns
        -------
        fig : Plotly graph_objects.Figure()
            A figure with the histogram plots.

        Examples
        --------
        >>> import ross.stochastic as srs
        >>> elm = srs.st_pointmass_example()
        >>> fig = elm.plot_random_var(["mx"])
        >>> # fig.show()
        """
        label = dict(
            mx="Mass on the X direction",
            my="Mass on the Y direction",
            m="Mass",
        )
        if var_list is None:
            var_list = self.is_random
        elif not all(var in self.is_random for var in var_list):
            raise ValueError(
                "Random variable not in var_list. Select variables from {}".format(
                    self.is_random
                )
            )

        return plot_histogram(
            self.attribute_dict, label, var_list, histogram_kwargs={}, plot_kwargs={}
        )


def st_pointmass_example():
    """Return an instance of a simple random point mass.

    The purpose is to make available a simple model so that doctest can be
    written using it.

    Returns
    -------
    elm : ross.stochastic.ST_PointMass
        An instance of a random point mass element object.

    Examples
    --------
    >>> import ross.stochastic as srs
    >>> elm = srs.st_pointmass_example()
    >>> len(list(iter(elm)))
    2
    """
    mx = [2.0, 2.5]
    my = [3.0, 3.5]
    elm = ST_PointMass(n=1, mx=mx, my=my, is_random=["mx", "my"])
    return elm



"""Bearing element module for STOCHASTIC ROSS.

This module creates an instance of random bearing for stochastic analysis.
"""

import numpy as np

from ross.bearing_seal_element import BearingElement
from ross.fluid_flow import fluid_flow as flow
from ross.fluid_flow.fluid_flow_coefficients import (
    calculate_stiffness_and_damping_coefficients,
)
from ross.stochastic.st_results_elements import plot_histogram
from ross.units import check_units

__all__ = ["ST_BearingElement", "st_bearing_example"]


class ST_BearingElement:
    """Random bearing element.

    Creates an object containing a list with random instances of
    BearingElement.

    Considering constant coefficients, use an 1-D array to make it random.
    Considering varying coefficients to the frequency, use a 2-D array to
    make it random (*see the Examples below).

    Parameters
    ----------
    n: int
        Node which the bearing will be located in
    kxx: float, 1-D array, 2-D array
        Direct stiffness in the x direction.
    cxx: float, 1-D array, 2-D array
        Direct damping in the x direction.
    kyy: float, 1-D array, 2-D array, optional
        Direct stiffness in the y direction.
        (defaults to kxx)
    kxy: float, 1-D array, 2-D array, optional
        Cross coupled stiffness in the x direction.
        (defaults to 0)
    kyx: float, 1-D array, 2-D array, optional
        Cross coupled stiffness in the y direction.
        (defaults to 0)
    cyy: float, 1-D array, 2-D array, optional
        Direct damping in the y direction.
        (defaults to cxx)
    cxy: float, 1-D array, 2-D array, optional
        Cross coupled damping in the x direction.
        (defaults to 0)
    cyx: float, 1-D array, 2-D array, optional
        Cross coupled damping in the y direction.
        (defaults to 0)
    frequency: array, optional
        Array with the frequencies (rad/s).
    tag: str, optional
        A tag to name the element
        Default is None.
    n_link: int, optional
        Node to which the bearing will connect. If None the bearing is
        connected to ground.
        Default is None.
    scale_factor: float, optional
        The scale factor is used to scale the bearing drawing.
        Default is 1.
    is_random : list
        List of the object attributes to become stochastic.
        Possibilities:
            ["kxx", "kxy", "kyx", "kyy", "cxx", "cxy", "cyx", "cyy"]

    Attributes
    ----------
    elements_list : list
        display the list with random bearing elements.

    Example
    -------
    >>> import numpy as np
    >>> import ross.stochastic as srs

    # Uncertanties on constant bearing coefficients

    >>> s = 10
    >>> kxx = np.random.uniform(1e6, 2e6, s)
    >>> cxx = np.random.uniform(1e3, 2e3, s)
    >>> elms = srs.ST_BearingElement(n=1,
    ...                              kxx=kxx,
    ...                              cxx=cxx,
    ...                              is_random = ["kxx", "cxx"],
    ...                              )
    >>> len(list(iter(elms)))
    10

    # Uncertanties on bearing coefficients varying with frequency

    >>> s = 5
    >>> kxx = [np.random.uniform(1e6, 2e6, s),
    ...        np.random.uniform(2.3e6, 3.3e6, s)]
    >>> cxx = [np.random.uniform(1e3, 2e3, s),
    ...        np.random.uniform(2.1e3, 3.1e3, s)]
    >>> frequency = np.linspace(500, 800, len(kxx))
    >>> elms = srs.ST_BearingElement(n=1,
    ...                              kxx=kxx,
    ...                              cxx=cxx,
    ...                              frequency=frequency,
    ...                              is_random = ["kxx", "cxx"],
    ...                              )
    >>> len(list(iter(elms)))
    5
    """

    @check_units
    def __init__(
        self,
        n,
        kxx,
        cxx,
        mxx=None,
        kyy=None,
        kxy=0,
        kyx=0,
        cyy=None,
        cxy=0,
        cyx=0,
        myy=None,
        mxy=0,
        myx=0,
        frequency=None,
        tag=None,
        n_link=None,
        scale_factor=1,
        is_random=None,
    ):
        if "frequency" in is_random:
            raise ValueError("frequency can not be a random variable")

        if kyy is None:
            kyy = kxx
            if "kxx" in is_random and "kyy" not in is_random:
                is_random.append("kyy")
        if cyy is None:
            cyy = cxx
            if "cxx" in is_random and "cyy" not in is_random:
                is_random.append("cyy")

        if myy is None:
            if mxx is None:
                myy = 0
                mxx = 0
            else:
                myy = mxx
            if "mxx" in is_random and "myy" not in is_random:
                is_random.append("myy")

        attribute_dict = dict(
            n=n,
            kxx=kxx,
            cxx=cxx,
            mxx=mxx,
            kyy=kyy,
            kxy=kxy,
            kyx=kyx,
            cyy=cyy,
            cxy=cxy,
            cyx=cyx,
            myy=myy,
            mxy=mxy,
            myx=myx,
            frequency=frequency,
            tag=tag,
            n_link=n_link,
            scale_factor=scale_factor,
        )
        self.is_random = is_random
        self.attribute_dict = attribute_dict

    def __iter__(self):
        """Return an iterator for the container.

        Returns
        -------
        An iterator over random bearing elements.

        Examples
        --------
        >>> import ross.stochastic as srs
        >>> bearing = srs.st_bearing_example()
        >>> len(list(iter(bearing)))
        2
        """
        return iter(self.random_var(self.is_random, self.attribute_dict))

    def __getitem__(self, key):
        """Return the value for a given key from attribute_dict.

        Parameters
        ----------
        key : str
            A class parameter as string.

        Raises
        ------
        KeyError
            Raises an error if the parameter doesn't belong to the class.

        Returns
        -------
        Return the value for the given key.

        Example
        -------
        >>> import numpy as np
        >>> import ross.stochastic as srs
        >>> s = 5
        >>> kxx = np.random.uniform(1e6, 2e6, s)
        >>> cxx = np.random.uniform(1e3, 2e3, s)
        >>> elms = srs.ST_BearingElement(n=1,
        ...                              kxx=kxx,
        ...                              cxx=cxx,
        ...                              is_random = ["kxx", "cxx"],
        ...                              )
        >>> elms["n"]
        1
        """
        if key not in self.attribute_dict.keys():
            raise KeyError("Object does not have parameter: {}.".format(key))

        return self.attribute_dict[key]

    def __setitem__(self, key, value):
        """Set new parameter values for the object.

        Function to change a parameter value.
        It's not allowed to add new parameters to the object.

        Parameters
        ----------
        key : str
            A class parameter as string.
        value : The corresponding value for the attrbiute_dict's key.
            ***check the correct type for each key in ST_BearingElement
            docstring.

        Raises
        ------
        KeyError
            Raises an error if the parameter doesn't belong to the class.

        Example
        -------
        >>> import numpy as np
        >>> import ross.stochastic as srs
        >>> s = 5
        >>> kxx = np.random.uniform(1e6, 2e6, s)
        >>> cxx = np.random.uniform(1e3, 2e3, s)
        >>> elms = srs.ST_BearingElement(n=1,
        ...                              kxx=kxx,
        ...                              cxx=cxx,
        ...                              is_random = ["kxx", "cxx"],
        ...                              )
        >>> elms["kxx"] = np.linspace(3e6, 5e6, 5)
        >>> elms["kxx"]
        array([3000000., 3500000., 4000000., 4500000., 5000000.])
        """
        if key not in self.attribute_dict.keys():
            raise KeyError("Object does not have parameter: {}.".format(key))
        self.attribute_dict[key] = value

    def random_var(self, is_random, *args):
        """Generate a list of objects as random attributes.

        This function creates a list of objects with random values for selected
        attributes from ross.BearingElement.

        Parameters
        ----------
        is_random : list
            List of the object attributes to become stochastic.
        *args : dict
            Dictionary instanciating the ross.BearingElement class.
            The attributes that are supposed to be stochastic should be
            set as lists of random variables.

        Returns
        -------
        f_list : generator
            Generator of random objects.
        """
        args_dict = args[0]
        new_args = []

        try:
            for i in range(len(args_dict[is_random[0]][0])):
                arg = []
                for key, value in args_dict.items():
                    if key in is_random:
                        arg.append([value[j][i] for j in range(len(value))])
                    else:
                        arg.append(value)
                new_args.append(arg)

        except TypeError:
            for i in range(len(args_dict[is_random[0]])):
                arg = []
                for key, value in args_dict.items():
                    if key in is_random:
                        arg.append(value[i])
                    else:
                        arg.append(value)
                new_args.append(arg)

        f_list = (BearingElement(*arg) for arg in new_args)

        return f_list

    def plot_random_var(self, var_list=None, histogram_kwargs=None, plot_kwargs=None):
        """Plot histogram and the PDF.

        This function creates a histogram to display the random variable
        distribution.

        Parameters
        ----------
        var_list : list, optional
            List of random variables, in string format, to plot.
            Default is plotting all the random variables.
        histogram_kwargs : dict, optional
            Additional key word arguments can be passed to change
            the plotly.go.histogram (e.g. histnorm="probability density", nbinsx=20...).
            *See Plotly API to more information.
        plot_kwargs : dict, optional
            Additional key word arguments can be passed to change the plotly go.figure
            (e.g. line=dict(width=4.0, color="royalblue"), opacity=1.0, ...).
            *See Plotly API to more information.

        Returns
        -------
        fig : Plotly graph_objects.Figure()
            A figure with the histogram plots.

        Examples
        --------
        >>> import ross.stochastic as srs
        >>> elm = srs.st_bearing_example()
        >>> fig = elm.plot_random_var(["kxx"])
        >>> # fig.show()
        """
        label = dict(
            kxx="Kxx",
            kxy="Kxy",
            kyx="Kyx",
            kyy="Kyy",
            cxx="Cxx",
            cxy="Cxy",
            cyx="Cyx",
            cyy="Cyy",
            mxx="Mxx",
            mxy="Mxy",
            myx="Myx",
            myy="Myy",
        )
        if var_list is None:
            var_list = self.is_random
        elif not all(var in self.is_random for var in var_list):
            raise ValueError(
                "Random variable not in var_list. Select variables from {}".format(
                    self.is_random
                )
            )

        return plot_histogram(
            self.attribute_dict, label, var_list, histogram_kwargs, plot_kwargs
        )

    @classmethod
    def from_fluid_flow(
        cls,
        n,
        nz,
        ntheta,
        length,
        omega,
        p_in,
        p_out,
        radius_rotor,
        radius_stator,
        viscosity,
        density,
        attitude_angle=None,
        eccentricity=None,
        load=None,
        omegap=None,
        immediately_calculate_pressure_matrix_numerically=True,
        tag=None,
        n_link=None,
        scale_factor=1,
        is_random=None,
    ):
        """Instantiate a bearing using inputs from its fluid flow.

        Parameters
        ----------
        n : int
            The node in which the bearing will be located in the rotor.
        is_random : list
            List of the object attributes to become random.
            Possibilities:
                ["length", "omega", "p_in", "p_out", "radius_rotor",
                 "radius_stator", "viscosity", "density", "attitude_angle",
                 "eccentricity", "load", "omegap"]
        tag: str, optional
            A tag to name the element
            Default is None.
        n_link: int, optional
            Node to which the bearing will connect. If None the bearing is
            connected to ground.
            Default is None.
        scale_factor: float, optional
            The scale factor is used to scale the bearing drawing.
            Default is 1.

        Grid related
        ^^^^^^^^^^^^
        Describes the discretization of the problem
        nz: int
            Number of points along the Z direction (direction of flow).
        ntheta: int
            Number of points along the direction theta. NOTE: ntheta must be odd.
        nradius: int
            Number of points along the direction r.
        length: float, list
            Length in the Z direction (m).
            Input a list to make it random.

        Operation conditions
        ^^^^^^^^^^^^^^^^^^^^
        Describes the operation conditions.
        omega: float, list
            Rotation of the rotor (rad/s).
            Input a list to make it random.
        p_in: float, list
            Input Pressure (Pa).
            Input a list to make it random.
        p_out: float, list
            Output Pressure (Pa).
            Input a list to make it random.
        load: float, list
            Load applied to the rotor (N).
            Input a list to make it random.

        Geometric data of the problem
        ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
        Describes the geometric data of the problem.
        radius_rotor: float, list
            Rotor radius (m).
            Input a list to make it random.
        radius_stator: float, list
            Stator Radius (m).
            Input a list to make it random.
        eccentricity: float, list
            Eccentricity (m) is the euclidean distance between rotor and stator
            centers.
            The center of the stator is in position (0,0).
            Input a list to make it random.

        Fluid characteristics
        ^^^^^^^^^^^^^^^^^^^^^
        Describes the fluid characteristics.
        viscosity: float, list
            Viscosity (Pa.s).
            Input a list to make it random.
        density: float, list
            Fluid density(Kg/m^3).
            Input a list to make it random.

        Returns
        -------
        random bearing: srs.ST_BearingElement
            A random bearing object.

        Examples
        --------
        >>> import numpy as np
        >>> import ross.stochastic as srs
        >>> nz = 8
        >>> ntheta = 64
        >>> length = 1.5 * 0.0254
        >>> omega = [157.1, 300]
        >>> p_in = 0.
        >>> p_out = 0.
        >>> radius_rotor = 3 * 0.0254
        >>> radius_stator = 3.003 * 0.0254
        >>> viscosity = np.random.uniform(2.4e-03, 2.8e-03, 5)
        >>> density = 860
        >>> load = 1244.1
        >>> elms = srs.ST_BearingElement.from_fluid_flow(
        ...     0, nz=nz, ntheta=ntheta, length=length,
        ...     omega=omega, p_in=p_in, p_out=p_out, radius_rotor=radius_rotor,
        ...     radius_stator=radius_stator, viscosity=viscosity, density=density,
        ...     load=load, is_random=["viscosity"]
        ... )
        >>> len(list(iter(elms)))
        5
        """
        attribute_dict = locals()
        attribute_dict.pop("cls")
        attribute_dict.pop("omega")
        attribute_dict.pop("is_random")
        size = len(attribute_dict[is_random[0]])
        args_dict = {
            "kxx": np.zeros((len(omega), size)),
            "kxy": np.zeros((len(omega), size)),
            "kyx": np.zeros((len(omega), size)),
            "kyy": np.zeros((len(omega), size)),
            "cxx": np.zeros((len(omega), size)),
            "cxy": np.zeros((len(omega), size)),
            "cyx": np.zeros((len(omega), size)),
            "cyy": np.zeros((len(omega), size)),
        }

        for k, v in attribute_dict.items():
            if k not in is_random:
                attribute_dict[k] = np.full(size, v)
            else:
                attribute_dict[k] = np.asarray(v)

        for j, frequency in enumerate(omega):
            for i in range(size):
                fluid_flow = flow.FluidFlow(
                    attribute_dict["nz"][i],
                    attribute_dict["ntheta"][i],
                    attribute_dict["length"][i],
                    frequency,
                    attribute_dict["p_in"][i],
                    attribute_dict["p_out"][i],
                    attribute_dict["radius_rotor"][i],
                    attribute_dict["radius_stator"][i],
                    attribute_dict["viscosity"][i],
                    attribute_dict["density"][i],
                    attribute_dict["attitude_angle"][i],
                    attribute_dict["eccentricity"][i],
                    attribute_dict["load"][i],
                    attribute_dict["omegap"][i],
                )
                k, c = calculate_stiffness_and_damping_coefficients(fluid_flow)

                args_dict["kxx"][j, i] = k[0]
                args_dict["kxy"][j, i] = k[1]
                args_dict["kyx"][j, i] = k[2]
                args_dict["kyy"][j, i] = k[3]
                args_dict["cxx"][j, i] = c[0]
                args_dict["cxy"][j, i] = c[1]
                args_dict["cyx"][j, i] = c[2]
                args_dict["cyy"][j, i] = c[3]

        return cls(
            n,
            kxx=args_dict["kxx"],
            cxx=args_dict["cxx"],
            kyy=args_dict["kyy"],
            kxy=args_dict["kxy"],
            kyx=args_dict["kyx"],
            cyy=args_dict["cyy"],
            cxy=args_dict["cxy"],
            cyx=args_dict["cyx"],
            frequency=omega,
            tag=tag,
            n_link=n_link,
            scale_factor=scale_factor,
            is_random=list(args_dict.keys()),
        )


def st_bearing_example():
    """Return an instance of a simple random bearing.

    The purpose is to make available a simple model so that doctest can be
    written using it.

    Returns
    -------
    elm : ross.stochastic.ST_BearingElement
        An instance of a random bearing element object.

    Examples
    --------
    >>> import ross.stochastic as srs
    >>> elm = srs.st_bearing_example()
    >>> len(list(iter(elm)))
    2
    """
    kxx = [1e6, 2e6]
    cxx = [1e3, 2e3]
    elm = ST_BearingElement(n=1, kxx=kxx, cxx=cxx, is_random=["kxx", "cxx"])

    return elm



import sys
from math import isnan

import numpy as np
from scipy import integrate
from scipy.optimize import least_squares

from ross.fluid_flow.fluid_flow_geometry import move_rotor_center, move_rotor_center_abs


def calculate_oil_film_force(fluid_flow_object, force_type=None):
    """This function calculates the forces of the oil film in the N and T directions, ie in the
    opposite direction to the eccentricity and in the tangential direction.
    Parameters
    ----------
    fluid_flow_object: A FluidFlow object.
    force_type: str
        If set, calculates the oil film force matrix analytically considering the chosen type: 'short' or 'long'.
        If set to 'numerical', calculates the oil film force numerically.
    Returns
    -------
    radial_force: float
        Force of the oil film in the opposite direction to the eccentricity direction.
    tangential_force: float
        Force of the oil film in the tangential direction
    f_x: float
        Components of forces in the x direction
    f_y: float
        Components of forces in the y direction
    Examples
    --------
    >>> from ross.fluid_flow.fluid_flow import fluid_flow_example
    >>> my_fluid_flow = fluid_flow_example()
    >>> calculate_oil_film_force(my_fluid_flow) # doctest: +ELLIPSIS
    (...
    """
    if force_type != "numerical" and (
        force_type == "short" or fluid_flow_object.bearing_type == "short_bearing"
    ):
        radial_force = (
            0.5
            * fluid_flow_object.viscosity
            * (fluid_flow_object.radius_rotor / fluid_flow_object.radial_clearance) ** 2
            * (fluid_flow_object.length**3 / fluid_flow_object.radius_rotor)
            * (
                (2 * fluid_flow_object.eccentricity_ratio**2 * fluid_flow_object.omega)
                / (1 - fluid_flow_object.eccentricity_ratio**2) ** 2
            )
        )

        tangential_force = (
            0.5
            * fluid_flow_object.viscosity
            * (fluid_flow_object.radius_rotor / fluid_flow_object.radial_clearance) ** 2
            * (fluid_flow_object.length**3 / fluid_flow_object.radius_rotor)
            * (
                (np.pi * fluid_flow_object.eccentricity_ratio * fluid_flow_object.omega)
                / (2 * (1 - fluid_flow_object.eccentricity_ratio**2) ** (3.0 / 2))
            )
        )
    elif force_type != "numerical" and (
        force_type == "long" or fluid_flow_object.bearing_type == "long_bearing"
    ):
        radial_force = (
            6
            * fluid_flow_object.viscosity
            * (fluid_flow_object.radius_rotor / fluid_flow_object.radial_clearance) ** 2
            * fluid_flow_object.radius_rotor
            * fluid_flow_object.length
            * (
                (2 * fluid_flow_object.eccentricity_ratio**2 * fluid_flow_object.omega)
                / (
                    (2 + fluid_flow_object.eccentricity_ratio**2)
                    * (1 - fluid_flow_object.eccentricity_ratio**2)
                )
            )
        )
        tangential_force = (
            6
            * fluid_flow_object.viscosity
            * (fluid_flow_object.radius_rotor / fluid_flow_object.radial_clearance) ** 2
            * fluid_flow_object.radius_rotor
            * fluid_flow_object.length
            * (
                (np.pi * fluid_flow_object.eccentricity_ratio * fluid_flow_object.omega)
                / (
                    (2 + fluid_flow_object.eccentricity_ratio**2)
                    * (1 - fluid_flow_object.eccentricity_ratio**2) ** 0.5
                )
            )
        )
    else:
        p_mat = fluid_flow_object.p_mat_numerical
        a = np.zeros([fluid_flow_object.nz, fluid_flow_object.ntheta])
        b = np.zeros([fluid_flow_object.nz, fluid_flow_object.ntheta])
        g1 = np.zeros(fluid_flow_object.nz)
        g2 = np.zeros(fluid_flow_object.nz)
        base_vector = np.array(
            [
                fluid_flow_object.xre[0][0] - fluid_flow_object.xi,
                fluid_flow_object.yre[0][0] - fluid_flow_object.yi,
            ]
        )
        for i in range(fluid_flow_object.nz):
            for j in range(int(fluid_flow_object.ntheta)):
                vector_from_rotor = np.array(
                    [
                        fluid_flow_object.xre[i][j] - fluid_flow_object.xi,
                        fluid_flow_object.yre[i][j] - fluid_flow_object.yi,
                    ]
                )
                angle_between_vectors = np.arctan2(
                    vector_from_rotor[1], vector_from_rotor[0]
                ) - np.arctan2(base_vector[1], base_vector[0])
                if angle_between_vectors < 0:
                    angle_between_vectors += 2 * np.pi
                a[i][j] = p_mat[i][j] * np.cos(angle_between_vectors)
                b[i][j] = p_mat[i][j] * np.sin(angle_between_vectors)

        for i in range(fluid_flow_object.nz):
            g1[i] = integrate.simpson(a[i][:], x=fluid_flow_object.gama[0])
            g2[i] = integrate.simpson(b[i][:], x=fluid_flow_object.gama[0])

        integral1 = integrate.simpson(g1, x=fluid_flow_object.z_list)
        integral2 = integrate.simpson(g2, x=fluid_flow_object.z_list)

        angle_corr = (
            np.pi / 2
            - np.arctan2(base_vector[1], base_vector[0])
            + fluid_flow_object.attitude_angle
        )
        radial_force_aux = fluid_flow_object.radius_rotor * integral1
        tangential_force_aux = fluid_flow_object.radius_rotor * integral2
        radial_force = radial_force_aux * np.cos(
            angle_corr + np.pi
        ) + tangential_force_aux * np.cos(angle_corr + np.pi / 2)
        tangential_force = radial_force_aux * np.cos(
            angle_corr + np.pi / 2
        ) + tangential_force_aux * np.cos(angle_corr)

    force_x = -radial_force * np.sin(
        fluid_flow_object.attitude_angle
    ) + tangential_force * np.cos(fluid_flow_object.attitude_angle)
    force_y = radial_force * np.cos(
        fluid_flow_object.attitude_angle
    ) + tangential_force * np.sin(fluid_flow_object.attitude_angle)
    return radial_force, tangential_force, force_x, force_y


def calculate_stiffness_and_damping_coefficients(fluid_flow_object):
    """This function calculates the bearing stiffness and damping matrices numerically.
    Parameters
    ----------
    fluid_flow_object: A FluidFlow object.
    Returns
    -------
    Two lists of floats
        A list of length four including stiffness floats in this order: kxx, kxy, kyx, kyy.
        And another list of length four including damping floats in this order: cxx, cxy, cyx, cyy.
        And
    Examples
    --------
    >>> from ross.fluid_flow.fluid_flow import fluid_flow_example
    >>> my_fluid_flow = fluid_flow_example()
    >>> calculate_stiffness_and_damping_coefficients(my_fluid_flow)  # doctest: +ELLIPSIS
    ([4...
    """
    N = 6
    t = np.linspace(0, 2 * np.pi / fluid_flow_object.omegap, N)
    fluid_flow_object.xp = fluid_flow_object.radial_clearance * 0.0001
    fluid_flow_object.yp = fluid_flow_object.radial_clearance * 0.0001
    dx = np.zeros(N)
    dy = np.zeros(N)
    xdot = np.zeros(N)
    ydot = np.zeros(N)
    radial_force = np.zeros(N)
    tangential_force = np.zeros(N)
    force_xx = np.zeros(N)
    force_yx = np.zeros(N)
    force_xy = np.zeros(N)
    force_yy = np.zeros(N)
    X1 = np.zeros([N, 3])
    X2 = np.zeros([N, 3])
    F1 = np.zeros(N)
    F2 = np.zeros(N)
    F3 = np.zeros(N)
    F4 = np.zeros(N)

    for i in range(N):
        fluid_flow_object.t = t[i]

        delta_x = fluid_flow_object.xp * np.sin(
            fluid_flow_object.omegap * fluid_flow_object.t
        )
        move_rotor_center(fluid_flow_object, delta_x, 0)
        dx[i] = delta_x
        xdot[i] = (
            fluid_flow_object.omegap
            * fluid_flow_object.xp
            * np.cos(fluid_flow_object.omegap * fluid_flow_object.t)
        )
        fluid_flow_object.geometry_description()
        fluid_flow_object.calculate_pressure_matrix_numerical(direction="x")
        [
            radial_force[i],
            tangential_force[i],
            force_xx[i],
            force_yx[i],
        ] = calculate_oil_film_force(fluid_flow_object, force_type="numerical")

        delta_y = fluid_flow_object.yp * np.sin(
            fluid_flow_object.omegap * fluid_flow_object.t
        )
        move_rotor_center(fluid_flow_object, -delta_x, 0)
        move_rotor_center(fluid_flow_object, 0, delta_y)
        dy[i] = delta_y
        ydot[i] = (
            fluid_flow_object.omegap
            * fluid_flow_object.yp
            * np.cos(fluid_flow_object.omegap * fluid_flow_object.t)
        )
        fluid_flow_object.geometry_description()
        fluid_flow_object.calculate_pressure_matrix_numerical(direction="y")
        [
            radial_force[i],
            tangential_force[i],
            force_xy[i],
            force_yy[i],
        ] = calculate_oil_film_force(fluid_flow_object, force_type="numerical")
        move_rotor_center(fluid_flow_object, 0, -delta_y)
        fluid_flow_object.geometry_description()
        fluid_flow_object.calculate_pressure_matrix_numerical()

        X1[i] = [1, dx[i], xdot[i]]
        X2[i] = [1, dy[i], ydot[i]]
        F1[i] = -force_xx[i]
        F2[i] = -force_xy[i]
        F3[i] = -force_yx[i]
        F4[i] = -force_yy[i]

    P1 = np.dot(
        np.dot(np.linalg.inv(np.dot(np.transpose(X1), X1)), np.transpose(X1)), F1
    )
    P2 = np.dot(
        np.dot(np.linalg.inv(np.dot(np.transpose(X2), X2)), np.transpose(X2)), F2
    )
    P3 = np.dot(
        np.dot(np.linalg.inv(np.dot(np.transpose(X1), X1)), np.transpose(X1)), F3
    )
    P4 = np.dot(
        np.dot(np.linalg.inv(np.dot(np.transpose(X2), X2)), np.transpose(X2)), F4
    )

    K = [P1[1], P2[1], P3[1], P4[1]]
    C = [P1[2], P2[2], P3[2], P4[2]]

    return K, C


def calculate_short_stiffness_matrix(fluid_flow_object):
    """This function calculates the stiffness matrix for the short bearing.
    Parameters
    ----------
    fluid_flow_object: A FluidFlow object.
    Returns
    -------
    list of floats
        A list of length four including stiffness floats in this order: kxx, kxy, kyx, kyy
    Examples
    --------
    >>> from ross.fluid_flow.fluid_flow import fluid_flow_example
    >>> my_fluid_flow = fluid_flow_example()
    >>> calculate_short_stiffness_matrix(my_fluid_flow)  # doctest: +ELLIPSIS
    [417...
    """
    h0 = 1.0 / (
        (
            (np.pi**2) * (1 - fluid_flow_object.eccentricity_ratio**2)
            + 16 * fluid_flow_object.eccentricity_ratio**2
        )
        ** 1.5
    )
    a = fluid_flow_object.load / fluid_flow_object.radial_clearance
    kxx = (
        a
        * h0
        * 4
        * (
            (np.pi**2) * (2 - fluid_flow_object.eccentricity_ratio**2)
            + 16 * fluid_flow_object.eccentricity_ratio**2
        )
    )
    kxy = (
        a
        * h0
        * np.pi
        * (
            (np.pi**2) * (1 - fluid_flow_object.eccentricity_ratio**2) ** 2
            - 16 * fluid_flow_object.eccentricity_ratio**4
        )
        / (
            fluid_flow_object.eccentricity_ratio
            * np.sqrt(1 - fluid_flow_object.eccentricity_ratio**2)
        )
    )
    kyx = (
        -a
        * h0
        * np.pi
        * (
            (np.pi**2)
            * (1 - fluid_flow_object.eccentricity_ratio**2)
            * (1 + 2 * fluid_flow_object.eccentricity_ratio**2)
            + (32 * fluid_flow_object.eccentricity_ratio**2)
            * (1 + fluid_flow_object.eccentricity_ratio**2)
        )
        / (
            fluid_flow_object.eccentricity_ratio
            * np.sqrt(1 - fluid_flow_object.eccentricity_ratio**2)
        )
    )
    kyy = (
        a
        * h0
        * 4
        * (
            (np.pi**2) * (1 + 2 * fluid_flow_object.eccentricity_ratio**2)
            + (
                (32 * fluid_flow_object.eccentricity_ratio**2)
                * (1 + fluid_flow_object.eccentricity_ratio**2)
            )
            / (1 - fluid_flow_object.eccentricity_ratio**2)
        )
    )
    return [kxx, kxy, kyx, kyy]


def calculate_short_damping_matrix(fluid_flow_object):
    """This function calculates the damping matrix for the short bearing.
    Parameters
    -------
    fluid_flow_object: A FluidFlow object.
    Returns
    -------
    list of floats
        A list of length four including damping floats in this order: cxx, cxy, cyx, cyy
    Examples
    --------
    >>> from ross.fluid_flow.fluid_flow import fluid_flow_example
    >>> my_fluid_flow = fluid_flow_example()
    >>> calculate_short_damping_matrix(my_fluid_flow) # doctest: +ELLIPSIS
    [...
    """
    # fmt: off
    h0 = 1.0 / (((np.pi ** 2) * (1 - fluid_flow_object.eccentricity_ratio ** 2)
                 + 16 * fluid_flow_object.eccentricity_ratio ** 2) ** 1.5)
    a = fluid_flow_object.load / (fluid_flow_object.radial_clearance * fluid_flow_object.omega)
    cxx = (a * h0 * 2 * np.pi * np.sqrt(1 - fluid_flow_object.eccentricity_ratio ** 2) *
           ((np.pi ** 2) * (1 + 2 * fluid_flow_object.eccentricity_ratio ** 2)
            - 16 * fluid_flow_object.eccentricity_ratio ** 2) / fluid_flow_object.eccentricity_ratio)
    cxy = (-a * h0 * 8 * ((np.pi ** 2) * (1 + 2 * fluid_flow_object.eccentricity_ratio ** 2)
                          - 16 * fluid_flow_object.eccentricity_ratio ** 2))
    cyx = cxy
    cyy = (a * h0 * (2 * np.pi * (
            (np.pi ** 2) * (1 - fluid_flow_object.eccentricity_ratio ** 2) ** 2
            + 48 * fluid_flow_object.eccentricity_ratio ** 2)) /
           (fluid_flow_object.eccentricity_ratio * np.sqrt(1 - fluid_flow_object.eccentricity_ratio ** 2)))
    # fmt: on
    return [cxx, cxy, cyx, cyy]


def find_equilibrium_position(fluid_flow_object, print_equilibrium_position=False):
    """This function finds the equilibrium position of the rotor such that the fluid flow
    forces match the applied load.
    Parameters
    ----------
    fluid_flow_object: A FluidFlow object.
    print_equilibrium_position: bool, optional
        If True, prints the equilibrium position.
    Returns
    -------
    None
    --------
    >>> from ross.fluid_flow.fluid_flow import fluid_flow_example2
    >>> my_fluid_flow = fluid_flow_example2()
    >>> find_equilibrium_position(my_fluid_flow)
    >>> (my_fluid_flow.xi, my_fluid_flow.yi) # doctest: +ELLIPSIS
    (2.2...
    """

    def residuals(x, *args):
        """Calculates x component of the forces of the oil film and the
        difference between the y component and the load.
        Parameters
        ----------
        x: array
            Rotor center coordinates
        *args : dict
            Dictionary instantiating the ross.FluidFlow class.
        Returns
        -------
        array
            Array with the x component of the forces of the oil film and the difference
            between the y component and the load.
        """
        bearing = args[0]
        move_rotor_center_abs(
            bearing,
            x[0] * fluid_flow_object.radial_clearance,
            x[1] * fluid_flow_object.radial_clearance,
        )
        bearing.geometry_description()
        bearing.calculate_pressure_matrix_numerical()
        (_, _, fx, fy) = calculate_oil_film_force(bearing, force_type="numerical")
        return np.array([fx, (fy - bearing.load)])

    if fluid_flow_object.load is None:
        sys.exit("Load must be given to calculate the equilibrium position.")
    x0 = np.array(
        [
            0 * fluid_flow_object.radial_clearance,
            -1e-3 * fluid_flow_object.radial_clearance,
        ]
    )
    move_rotor_center_abs(fluid_flow_object, x0[0], x0[1])
    fluid_flow_object.geometry_description()
    fluid_flow_object.calculate_pressure_matrix_numerical()
    (_, _, fx, fy) = calculate_oil_film_force(fluid_flow_object, force_type="numerical")
    result = least_squares(
        residuals, x0, args=[fluid_flow_object], jac="3-point", bounds=([0, -1], [1, 0])
    )
    move_rotor_center_abs(
        fluid_flow_object,
        result.x[0] * fluid_flow_object.radial_clearance,
        result.x[1] * fluid_flow_object.radial_clearance,
    )
    fluid_flow_object.geometry_description()
    if print_equilibrium_position is True:
        print(
            "The equilibrium position (x0, y0) is: (",
            result.x[0] * fluid_flow_object.radial_clearance,
            ",",
            result.x[1] * fluid_flow_object.radial_clearance,
            ")",
        )



import inspect
from abc import ABC, abstractmethod
from collections import namedtuple
from pathlib import Path

import pandas as pd
import toml


class Element(ABC):
    """Element class.

    This class is a general abstract class to be implemented in other files, in order to
    create specific elements for the user.
    """

    def __init__(self, n, tag=None):
        self.n = n
        self.tag = tag

    def save(self, file):
        """Save the element in a .toml file.

        This function will save the element to a .toml file.
        The file will have all the argument's names and values that are needed to
        reinstantiate the element.

        Parameters
        ----------
        file : str, pathlib.Path
            The name of the file the element will be saved in.

        Examples
        --------
        >>> # Example using DiskElement
        >>> from tempfile import tempdir
        >>> from pathlib import Path
        >>> from ross.disk_element import disk_example
        >>> # create path for a temporary file
        >>> file = Path(tempdir) / 'disk.toml'
        >>> disk = disk_example()
        >>> disk.save(file)
        """
        # get __init__ arguments
        signature = inspect.signature(self.__init__)
        args_list = list(signature.parameters)
        args = {arg: getattr(self, arg) for arg in args_list}
        try:
            data = toml.load(file)
        except FileNotFoundError:
            data = {}

        data[f"{self.__class__.__name__}_{self.tag}"] = args
        with open(file, "w") as f:
            toml.dump(data, f)

    @classmethod
    def read_toml_data(cls, data):
        """Read and parse data stored in a .toml file.

        The data passed to this method needs to be according to the
        format saved in the .toml file by the .save() method.

        Parameters
        ----------
        data : dict
            Dictionary obtained from toml.load().

        Returns
        -------
        The element object.

        Examples
        --------
        >>> # Example using BearingElement
        >>> from tempfile import tempdir
        >>> from pathlib import Path
        >>> from ross.bearing_seal_element import bearing_example
        >>> from ross.bearing_seal_element import BearingElement
        >>> # create path for a temporary file
        >>> file = Path(tempdir) / 'bearing1.toml'
        >>> bearing1 = bearing_example()
        >>> bearing1.save(file)
        >>> bearing1_loaded = BearingElement.load(file)
        >>> bearing1 == bearing1_loaded
        True
        """
        return cls(**data)

    @classmethod
    def load(cls, file):
        data = toml.load(file)
        # extract single dictionary in the data
        data = list(data.values())[0]
        return cls.read_toml_data(data)

    @abstractmethod
    def M(self):
        """Mass matrix.

        Returns
        -------
        A matrix of floats.

        Examples
        --------
        >>> # Example using BearingElement
        >>> from ross.bearing_seal_element import bearing_example
        >>> bearing = bearing_example()
        >>> bearing.M(0)
        array([[0., 0.],
               [0., 0.]])
        """
        pass

    @abstractmethod
    def C(self, frequency):
        """Frequency dependent damping coefficients matrix.

        Parameters
        ----------
        frequency: float
            The frequency in which the coefficients depend on.

        Returns
        -------
        A matrix of floats.

        Examples
        --------
        >>> # Example using BearingElement
        >>> from ross.bearing_seal_element import bearing_example
        >>> bearing = bearing_example()
        >>> bearing.C(0)
        array([[200.,   0.],
               [  0., 150.]])
        """
        pass

    @abstractmethod
    def K(self, frequency):
        """Frequency dependent stiffness coefficients matrix.

        Parameters
        ----------
        frequency: float
            The frequency in which the coefficients depend on.

        Returns
        -------
        A matrix of floats.

        Examples
        --------
        >>> # Example using BearingElement
        >>> from ross.bearing_seal_element import bearing_example
        >>> bearing = bearing_example()
        >>> bearing.K(0)
        array([[1000000.,       0.],
               [      0.,  800000.]])
        """
        pass

    @abstractmethod
    def G(self):
        """Gyroscopic matrix.

        Returns
        -------
        A matrix of floats.

        Examples
        --------
        >>> # Example using BearingElement
        >>> from ross.bearing_seal_element import bearing_example
        >>> bearing = bearing_example()
        >>> bearing.G()
        array([[0., 0.],
               [0., 0.]])
        """
        pass

    def summary(self):
        """Present a summary for the element.

        A pandas series with the element properties as variables.

        Returns
        -------
        A pandas series.

        Examples
        --------
        >>> # Example using DiskElement
        >>> from ross.disk_element import disk_example
        >>> disk = disk_example()
        >>> disk.summary() # doctest: +ELLIPSIS
        n                             0
        n_l                           0
        n_r                           0...
        """
        attributes = self.__dict__
        attributes["type"] = self.__class__.__name__
        return pd.Series(attributes)

    @abstractmethod
    def dof_mapping(self):
        """Degrees of freedom mapping.

        Should return a dictionary with a mapping between degree of freedom
        and its index.

        Returns
        -------
        dof_mapping: dict
            A dictionary containing the degrees of freedom and their indexes.

        Examples
        --------
        >>> # Example using BearingElement
        >>> from ross.bearing_seal_element import bearing_example
        >>> bearing = bearing_example()
        >>> bearing.dof_mapping()
        {'x_0': 0, 'y_0': 1}
        """
        pass

    def dof_local_index(self):
        """Get the local index for a element specific degree of freedom.

        Returns
        -------
        local_index: namedtupple
            A named tuple containing the local index.

        Examples
        --------
        >>> # Example using BearingElement
        >>> from ross.bearing_seal_element import bearing_example
        >>> bearing = bearing_example()
        >>> bearing.dof_local_index()
        LocalIndex(x_0=0, y_0=1)
        """
        dof_mapping = self.dof_mapping()
        dof_tuple = namedtuple("LocalIndex", dof_mapping)
        local_index = dof_tuple(**dof_mapping)

        return local_index



import pytest
from numpy.testing import assert_allclose
import numpy as np
import ross as rs


@pytest.fixture
def rotor_4dof():
    return rs.rotor_example()


@pytest.fixture
def rotor_6dof(rotor_4dof):
    shaft_elem = [
        rs.ShaftElement6DoF(
            material=rotor_4dof.shaft_elements[l].material,
            L=rotor_4dof.shaft_elements[l].L,
            n=rotor_4dof.shaft_elements[l].n,
            idl=rotor_4dof.shaft_elements[l].idl,
            odl=rotor_4dof.shaft_elements[l].odl,
            idr=rotor_4dof.shaft_elements[l].idr,
            odr=rotor_4dof.shaft_elements[l].odr,
        )
        for l, p in enumerate(rotor_4dof.shaft_elements)
    ]

    disk_elem = [
        rs.DiskElement6DoF(
            n=rotor_4dof.disk_elements[l].n,
            m=rotor_4dof.disk_elements[l].m,
            Id=rotor_4dof.disk_elements[l].Id,
            Ip=rotor_4dof.disk_elements[l].Ip,
        )
        for l in range(len(rotor_4dof.disk_elements))
    ]

    bearing_elem = [
        rs.BearingElement6DoF(
            n=rotor_4dof.bearing_elements[l].n,
            kxx=rotor_4dof.bearing_elements[l].kxx,
            kyy=rotor_4dof.bearing_elements[l].kyy,
            cxx=rotor_4dof.bearing_elements[l].cxx,
            cyy=rotor_4dof.bearing_elements[l].cyy,
            kxy=rotor_4dof.bearing_elements[l].kxy,
            kyx=rotor_4dof.bearing_elements[l].kyx,
            cxy=rotor_4dof.bearing_elements[l].cxy,
            cyx=rotor_4dof.bearing_elements[l].cyx,
            frequency=rotor_4dof.bearing_elements[l].frequency,
        )
        for l in range(len(rotor_4dof.bearing_elements))
    ]

    return rs.Rotor(shaft_elem, disk_elem, bearing_elem)


def test_run_static(rotor_6dof):
    static = rotor_6dof.run_static()

    bearing_forces = {"node_0": 432.3774305443053, "node_6": 432.3774305443162}
    disk_forces = {"node_2": 319.59116422953997, "node_4": 319.59116422953997}

    # fmt:off
    deformation = np.array([
        -4.32377431e-18, -3.73948262e-04, -6.48759479e-04, -7.45972747e-04,
        -6.48759479e-04, -3.73948262e-04, -4.32377431e-18
    ])
    # fmt:on

    for n in bearing_forces:
        assert_allclose(static.bearing_forces[n], bearing_forces[n], rtol=1e-6)

    for n in disk_forces:
        assert_allclose(static.disk_forces[n], disk_forces[n], rtol=1e-6)

    assert_allclose(static.deformation, deformation, rtol=1e-6)


def test_static_results_equality(rotor_6dof, rotor_4dof):
    static1 = rotor_6dof.run_static()
    static2 = rotor_4dof.run_static()

    for n in static1.bearing_forces:
        assert_allclose(static1.bearing_forces[n], static2.bearing_forces[n], rtol=1e-6)

    for n in static1.disk_forces:
        assert_allclose(static1.disk_forces[n], static2.disk_forces[n], rtol=1e-6)

    assert_allclose(static1.deformation, static2.deformation, rtol=1e-6)


def test_run_modal(rotor_6dof):
    speed = 100.0
    modal = rotor_6dof.run_modal(speed, num_modes=14)

    # fmt:off
    wn = np.array([
         91.78677 ,  96.29604 , 274.059892, 296.994125, 717.35166 ,
        770.203624, 774.349678
    ])
    wd = np.array([
         91.78677 ,  96.29604 , 274.059892, 296.994125, 717.35166 ,
        770.203624, 774.349678
    ])
    # fmt:on

    assert_allclose(modal.wn, wn, rtol=5e-2, atol=1)
    assert_allclose(modal.wd, wd, rtol=5e-2, atol=1)


def test_modal_results_equality(rotor_6dof, rotor_4dof):
    speed = 100.0
    modal_6dof = rotor_6dof.run_modal(speed, num_modes=10)
    modal_4dof = rotor_4dof.run_modal(speed, num_modes=10)

    assert_allclose(modal_6dof.wn, modal_4dof.wn, rtol=5e-2, atol=1)
    assert_allclose(modal_6dof.wd, modal_4dof.wd, rtol=5e-2, atol=1)


def test_campbell(rotor_6dof):
    speed_range = np.linspace(315, 1150, 31)
    campbell = rotor_6dof.run_campbell(speed_range)

    # fmt:off
    wd = np.array([
        91.70122235, 91.68341248, 91.66467645, 91.64459554, 91.62319309,
        91.60049282, 91.57652003, 91.55129967, 91.52485766, 91.49721995,
        91.46841277, 91.43846178, 91.40739326, 91.37523348, 91.34200733,
        91.30774144, 91.27245973, 91.23618811, 91.19894932, 91.1607676 ,
        91.12166632, 91.08166769, 91.04079376, 90.99906592, 90.95650489,
        90.91313073, 90.86896298, 90.82402031, 90.77832143, 90.73188463,
        90.68472497
    ])
    # fmt:on

    assert_allclose(campbell.wd[:, 0], wd, rtol=1e-3)


def test_campbell_equality(rotor_6dof, rotor_4dof):
    speed_range = np.linspace(315, 1150, 31)
    campbell1 = rotor_6dof.run_campbell(speed_range)
    campbell2 = rotor_4dof.run_campbell(speed_range)

    assert_allclose(campbell1.wd[:, 0], campbell2.wd[:, 0], rtol=1e-3)


def test_run_freq(rotor_6dof):
    speed_range = np.linspace(315, 1150, 31)
    response = rotor_6dof.run_freq_response(speed_range=speed_range)

    # fmt:off
    abs_resp = np.array([
        [2.48335649e-06, 8.84174113e-07],
        [8.84174113e-07, 1.07819686e-06]
    ])
    ang_resp = np.array([
        [3.14159265, 1.57079633],
        [1.57079633, 3.14159265]
    ])
    # fmt:on

    assert_allclose(abs(response.freq_resp[:2, :2, 0]), abs_resp, atol=1e-7)
    assert_allclose(abs(np.angle(response.freq_resp[:2, :2, 0])), ang_resp, atol=1e-7)


def test_freq_resp_equality(rotor_6dof, rotor_4dof):
    speed_range = np.linspace(315, 1150, 31)
    response1 = rotor_6dof.run_freq_response(speed_range=speed_range)
    response2 = rotor_4dof.run_freq_response(speed_range=speed_range)

    assert_allclose(
        abs(response1.freq_resp[:2, :2, :]),
        abs(response2.freq_resp[:2, :2, :]),
        atol=1e-7,
    )


def test_run_unb(rotor_6dof):
    speed = np.linspace(0, 100, 31)
    response = rotor_6dof.run_unbalance_response(
        node=3, unbalance_magnitude=10.0, unbalance_phase=0.0, frequency=speed
    )

    abs_resp = np.array([0.01763464, 0.02290302])
    ang_resp = np.array([1.18722483e-12, -1.57079633e00])

    assert_allclose(abs(response.forced_resp[:2, 15]), abs_resp, atol=1e-7)
    assert_allclose(np.angle(response.forced_resp[:2, 15]), ang_resp, atol=1e-7)


def test_unb_resp_equality(rotor_6dof, rotor_4dof):
    speed = np.linspace(0, 100, 31)
    response1 = rotor_6dof.run_unbalance_response(
        node=3, unbalance_magnitude=10.0, unbalance_phase=0.0, frequency=speed
    )
    response2 = rotor_4dof.run_unbalance_response(
        node=3, unbalance_magnitude=10.0, unbalance_phase=0.0, frequency=speed
    )

    assert_allclose(
        abs(response1.forced_resp[:2, :]), abs(response2.forced_resp[:2, :]), atol=1e-7
    )


def input_run_time(rotor):
    speed = 500.0
    size = 1000
    node = 3
    t = np.linspace(0, 10, size)
    F = np.zeros((size, rotor.ndof))
    F[:, rotor.number_dof * node + 0] = 10 * np.cos(2 * t)
    F[:, rotor.number_dof * node + 1] = 10 * np.sin(2 * t)

    return speed, F, t


def test_run_time(rotor_6dof):
    input_6dof = input_run_time(rotor_6dof)

    response1 = rotor_6dof.run_time_response(*input_6dof)
    response2 = rotor_6dof.run_time_response(*input_6dof, integrator="newmark")
    # fmt:off
    response3 = np.array([
        4.23256631e-06,  7.70160885e-06,  0.00000000e+00, -2.57207762e-05,
        2.17306960e-05,  0.00000000e+00,  9.46767650e-06,  1.38792218e-05,
        0.00000000e+00, -2.25172698e-05,  1.91808098e-05,  0.00000000e+00,
        1.34302952e-05,  1.84580413e-05,  0.00000000e+00, -1.30644256e-05,
        1.17692246e-05,  0.00000000e+00,  1.49418357e-05,  2.01980866e-05,
        0.00000000e+00,  1.33928884e-17, -6.43108349e-17,  0.00000000e+00,
        1.34302952e-05,  1.84580413e-05,  0.00000000e+00,  1.30644256e-05,
       -1.17692246e-05,  0.00000000e+00,  9.46767650e-06,  1.38792218e-05,
        0.00000000e+00,  2.25172698e-05, -1.91808098e-05,  0.00000000e+00,
        4.23256631e-06,  7.70160885e-06,  0.00000000e+00,  2.57207762e-05,
       -2.17306960e-05,  0.00000000e+00
    ])
    # fmt:on

    dof = 3 * 6 + 1
    assert_allclose(
        np.mean(response1.yout[:, dof]), np.mean(response2.yout[:, dof]), atol=1e-7
    )
    assert_allclose(response1.yout[50, :], response3, atol=1e-7)


def test_time_resp_equality(rotor_6dof, rotor_4dof):
    input_6dof = input_run_time(rotor_6dof)
    input_4dof = input_run_time(rotor_4dof)

    response1 = rotor_6dof.run_time_response(*input_6dof)
    response2 = rotor_4dof.run_time_response(*input_4dof)

    dof1 = 3 * 6 + 1
    dof2 = 3 * 4 + 1
    assert_allclose(response1.yout[:, dof1], response2.yout[:, dof2], atol=1e-7)



import pickle
from pathlib import Path
from tempfile import tempdir

import numpy as np
from numpy.testing import assert_allclose

from ross.point_mass import PointMass


def test_point_mass():
    # fmt: off
    m0 = np.array([[10.0, 0],
                   [0, 10.0]])
    # fmt: on

    p = PointMass(n=0, m=10.0, tag="pointmass")

    assert p.tag == "pointmass"
    assert_allclose(m0, p.M())
    assert_allclose(np.zeros((2, 2)), p.K())
    assert_allclose(np.zeros((2, 2)), p.C())
    assert_allclose(np.zeros((2, 2)), p.G())


def test_local_index():
    n = 0
    mx = 1.0
    my = 2.0
    p = PointMass(n=n, mx=mx, my=my)

    assert p.dof_local_index().x_0 == 0
    assert p.dof_local_index().y_0 == 1


def test_pickle():
    p = PointMass(n=0, m=10.0, tag="pointmass")
    p_pickled = pickle.loads(pickle.dumps(p))
    assert p == p_pickled


def test_save_load():
    p = PointMass(n=0, m=10.0, tag="pointmass")
    file = Path(tempdir) / "point_mass.toml"
    p.save(file)
    p_loaded = p.load(file)

    assert p == p_loaded



import math

import numpy as np
import pytest
from numpy.testing import assert_allclose, assert_almost_equal

from ross.fluid_flow.cylindrical import THDCylindrical
from ross.units import Q_


@pytest.fixture
def cylindrical():
    speed = Q_([900], "RPM")
    L = Q_(10.3600055944, "in")

    bearing = THDCylindrical(
        axial_length=L,
        journal_radius=0.2,
        radial_clearance=1.95e-4,
        elements_circumferential=11,
        elements_axial=3,
        n_pad=2,
        pad_arc_length=176,
        preload=0,
        geometry="circular",
        reference_temperature=50,
        speed=speed,
        load_x_direction=0,
        load_y_direction=-112814.91,
        groove_factor=[0.52, 0.48],
        lubricant="ISOVG32",
        node=3,
        sommerfeld_type=2,
        initial_guess=[0.1, -0.1],
        method="perturbation",
        operating_type="flooded",
        injection_pressure=0,
        oil_flow=37.86,
        show_coef=False,
        print_result=False,
        print_progress=False,
        print_time=False,
    )

    return bearing


def test_cylindrical_parameters(cylindrical):
    assert math.isclose(cylindrical.axial_length, 0.263144, rel_tol=0.0001)
    assert cylindrical.journal_radius == 0.2
    assert cylindrical.speed == 94.24777960769379
    assert cylindrical.rho == 873.99629
    assert cylindrical.reference_temperature == 50


def test_cylindrical_equilibrium_pos(cylindrical):
    assert math.isclose(cylindrical.equilibrium_pos[0], 0.68733194, rel_tol=0.01)
    assert math.isclose(cylindrical.equilibrium_pos[1], -0.79394211, rel_tol=0.01)


def test_cylindrical_coefficients(cylindrical):
    coefs = cylindrical.coefficients()
    kxx = coefs[0][0]
    kxy = coefs[0][1]
    kyx = coefs[0][2]
    kyy = coefs[0][3]
    cxx = coefs[1][0]
    cxy = coefs[1][1]
    cyx = coefs[1][2]
    cyy = coefs[1][3]

    assert math.isclose(kxx, 2497674531.1749372, rel_tol=0.0001)
    assert math.isclose(kxy, 783937669.6587772, rel_tol=0.0001)
    assert math.isclose(kyx, -3140562821.5290236, rel_tol=0.0001)
    assert math.isclose(kyy, 2562440911.734241, rel_tol=0.0001)
    assert math.isclose(cxx, 36950674.61976142, rel_tol=0.0001)
    assert math.isclose(cxy, -37265296.2322692, rel_tol=0.0001)
    assert math.isclose(cyx, -42642543.712838694, rel_tol=0.0001)
    assert math.isclose(cyy, 100992315.0043159, rel_tol=0.0001)



# Note: To use the 'upload' functionality of this file, you must:
#   $ pip install twine

import io
import os
import re

from setuptools import find_packages, setup


def read(path, encoding="utf-8"):
    path = os.path.join(os.path.dirname(__file__), path)
    with io.open(path, encoding=encoding) as fp:
        return fp.read()


def version(path):
    """Obtain the packge version from a python file e.g. pkg/__init__.py
    See <https://packaging.python.org/en/latest/single_source_version.html>.
    """
    version_file = read(path)
    version_match = re.search(
        r"""^__version__ = ['"]([^'"]*)['"]""", version_file, re.M
    )
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


# Package meta-data.
NAME = "ross-rotordynamics"
DESCRIPTION = "ROSS: Rotordynamic Open Source Software"
EMAIL = "raphaelts@petrobras.com.br"
AUTHOR = "ROSS developers"
REQUIRES_PYTHON = ">=3.7.0"
VERSION = version("ross/__init__.py")

# What packages are required for this module to be executed?
with open("requirements.txt") as f:
    REQUIRED = f.read().splitlines()

# What packages are optional?
EXTRAS = {
    "dev": [
        "pytest>=4.6",
        "pytest-cov",
        "coverage",
        "codecov",
        "sphinx",
        "myst-nb",
        "sphinx-book-theme",
        "sphinx-panels",
        "sphinx-copybutton",
        "sphinx-rtd-theme",
        "linkify-it-py",
        "numpydoc",
        "sphinxcontrib-bibtex>=2.2",
        "black",
        "isort",
        "sphinx-design",
    ]
}

# The rest you shouldn't have to touch too much :)
# ------------------------------------------------
# Except, perhaps the License and Trove Classifiers!
# If you do change the License, remember to change the Trove Classifier for that!

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
try:
    with io.open(os.path.join(here, "README.md"), encoding="utf-8") as f:
        long_description = "\n" + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Where the magic happens:
setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    project_urls={
        "Documentation": "https://ross.readthedocs.io/en/stable/",
        "Bug Tracker": "https://github.com/petrobras/ross/issues",
        "Discussions": "https://github.com/petrobras/ross/discussions",
        "Source Code": "https://github.com/petrobras/ross",
    },
    packages=find_packages(exclude=("tests",)),
    # If your package is a single module, use this instead of 'packages':
    # py_modules=['mypackage'],
    # entry_points={
    #     'console_scripts': ['mycli=mymodule:cli'],
    # },
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    package_data={"": ["new_units.txt"]},
    include_package_data=True,
    license="Apache License 2.0",
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
)



import pytest
from numpy.testing import assert_allclose
import numpy as np
from scipy import integrate

from ross.bearing_seal_element import BearingElement6DoF
from ross.disk_element import DiskElement6DoF
from ross.materials import steel
from ross.rotor_assembly import Rotor
from ross.shaft_element import ShaftElement6DoF


@pytest.fixture
def rotor1():
    #  Rotor with damping
    #  Rotor with 6 shaft elements, 2 disks and 2 bearings
    i_d = 0
    o_d = 0.05
    n = 6
    L = [0.25 for _ in range(n)]

    shaft_elem = [
        ShaftElement6DoF(
            l,
            i_d,
            o_d,
            material=steel,
            shear_effects=True,
            rotary_inertia=True,
            gyroscopic=True,
            alpha=10,
            beta=1e-4,
        )
        for l in L
    ]

    disk0 = DiskElement6DoF.from_geometry(2, steel, 0.07, 0.05, 0.28)
    disk1 = DiskElement6DoF.from_geometry(4, steel, 0.07, 0.05, 0.35)

    stfx = 1e6
    stfy = 1e6
    c = 1e3
    bearing0 = BearingElement6DoF(0, kxx=stfx, kyy=stfy, cxx=c, cyy=c)
    bearing1 = BearingElement6DoF(6, kxx=stfx, kyy=stfy, cxx=c, cyy=c)

    return Rotor(shaft_elem, [disk0, disk1], [bearing0, bearing1])


@pytest.fixture
def rotor2():
    #  Rotor with damping
    #  Rotor with 6 shaft elements, 2 disks and 2 bearings with frequency dependent coefficients
    i_d = 0
    o_d = 0.05
    n = 6
    L = [0.25 for _ in range(n)]

    shaft_elem = [
        ShaftElement6DoF(
            l,
            i_d,
            o_d,
            material=steel,
            shear_effects=True,
            rotary_inertia=True,
            gyroscopic=True,
            alpha=10,
            beta=1e-4,
        )
        for l in L
    ]

    disk0 = DiskElement6DoF.from_geometry(2, steel, 0.07, 0.05, 0.28)
    disk1 = DiskElement6DoF.from_geometry(4, steel, 0.07, 0.05, 0.35)

    stfx = [1e6, 1.5e6]
    stfy = [1e6, 1.5e6]
    c = [1e3, 1.5e3]
    frequency = [50, 5000]
    bearing0 = BearingElement6DoF(
        0, kxx=stfx, kyy=stfy, cxx=c, cyy=c, frequency=frequency
    )
    bearing1 = BearingElement6DoF(
        6, kxx=stfx, kyy=stfy, cxx=c, cyy=c, frequency=frequency
    )

    return Rotor(shaft_elem, [disk0, disk1], [bearing0, bearing1])


def simulation_parameters():
    dt = 5e-4
    t0 = 0
    tf = 3.5
    t = np.arange(t0, tf + dt, dt)

    step1 = np.where(t == 3.0)[0][0]
    step2 = len(t) - 1

    probe_node = 3
    number_dof = 6
    dofx = probe_node * number_dof
    dofy = dofx + 1

    probe_params = {"time": [step1, step2], "dofs": [dofx, dofy]}

    return t, probe_params


def unbalance_force(rotor, speed, t):
    nodes = [2, 4]
    mag_unb = [0.01, 0.01]
    phase_unb = [np.pi / 2, 0]

    num_dof = rotor.number_dof
    F_size = rotor.ndof

    F = np.zeros((len(t), F_size))

    speed_is_array = isinstance(speed, (list, tuple, np.ndarray))

    if speed_is_array:
        theta = integrate.cumulative_trapezoid(speed, t, initial=0)
    else:
        theta = speed * t

    for i, node in enumerate(nodes):
        phi = phase_unb[i] + theta

        Fx = mag_unb[i] * (speed**2) * np.cos(phi)
        Fy = mag_unb[i] * (speed**2) * np.sin(phi)

        F[:, node * num_dof + 0] += Fx
        F[:, node * num_dof + 1] += Fy

    return F


def test_for_cte_speed(rotor1, rotor2):
    # Test for constant speed
    t, probe_params = simulation_parameters()
    speed = 50.0

    F = unbalance_force(rotor1, speed, t)

    response1 = rotor1.run_time_response(speed, F, t, method="newmark")
    response2 = rotor2.run_time_response(speed, F, t, method="newmark")

    s0 = probe_params["time"][0]
    s1 = probe_params["time"][1]
    dofx = probe_params["dofs"][0]
    dofy = probe_params["dofs"][1]

    assert_allclose(response1.yout[s0:s1, dofx], response2.yout[s0:s1, dofx])
    assert_allclose(response1.yout[s0:s1, dofy], response2.yout[s0:s1, dofy])

    freq = 7.96813
    abs_max = 7.51498e-5

    change_sign = np.where(np.diff(np.sign(response1.yout[s0:s1, dofx])))[0]
    freq1 = 1 / (t[change_sign[2]] - t[change_sign[0]])
    abs_max1 = np.max(np.abs(response1.yout[s0:s1, dofx]))

    assert_allclose(freq, freq1, rtol=1e-3)
    assert_allclose(abs_max, abs_max1, rtol=1e-3)


def test_for_var_speed_1(rotor1):
    # Test for variable speed and bearings with fixed coefficients
    t, probe_params = simulation_parameters()
    speed = np.linspace(50, 500, len(t))

    F = unbalance_force(rotor1, speed, t)

    # Running with direct method
    resp_common = rotor1.run_time_response(speed, F, t)

    # Running with pseudo-modal method
    resp_pseudo_modal = rotor1.run_time_response(speed, F, t, num_modes=24)

    # Running with pseudo-modal method and add_to_RHS callable function
    def unb_force(step, **curr_state):
        return F[step, :]

    resp_add_to_RHS = rotor1.run_time_response(
        speed,
        np.zeros((len(t), rotor1.ndof)),
        t,
        num_modes=24,
        add_to_RHS=unb_force,
        progress_interval=5,
    )

    s0 = probe_params["time"][0]
    s1 = probe_params["time"][1]
    dofx = probe_params["dofs"][0]
    dofy = probe_params["dofs"][1]

    assert_allclose(
        resp_common.yout[s0:s1, dofx], resp_pseudo_modal.yout[s0:s1, dofx], rtol=0.1
    )
    assert_allclose(
        resp_common.yout[s0:s1, dofy], resp_pseudo_modal.yout[s0:s1, dofy], rtol=0.1
    )

    assert_allclose(
        resp_pseudo_modal.yout[s0:s1, dofx], resp_add_to_RHS.yout[s0:s1, dofx]
    )
    assert_allclose(
        resp_pseudo_modal.yout[s0:s1, dofy], resp_add_to_RHS.yout[s0:s1, dofy]
    )

    freq = 68.96552
    abs_max = 1.52977e-4

    change_sign = np.where(np.diff(np.sign(resp_common.yout[s0:s1, dofx])))[0]
    freq1 = 1 / (t[change_sign[2]] - t[change_sign[0]])
    abs_max1 = np.max(np.abs(resp_common.yout[s0:s1, dofx]))

    assert_allclose(freq, freq1, rtol=1e-3)
    assert_allclose(abs_max, abs_max1, rtol=1e-3)


def test_for_var_speed_2(rotor2):
    # Test for variable speed and bearings with frequency dependent coefficients
    t, probe_params = simulation_parameters()
    speed = np.linspace(50, 500, len(t))

    F = unbalance_force(rotor2, speed, t)

    # Running with direct method
    resp_common = rotor2.run_time_response(speed, F, t)

    # Running with pseudo-modal method
    resp_pseudo_modal = rotor2.run_time_response(speed, F, t, num_modes=24)

    # Running with pseudo-modal method and add_to_RHS callable function
    def unb_force(step, **curr_state):
        return F[step, :]

    resp_add_to_RHS = rotor2.run_time_response(
        speed,
        np.zeros((len(t), rotor2.ndof)),
        t,
        num_modes=24,
        add_to_RHS=unb_force,
        progress_interval=5,
    )

    s0 = probe_params["time"][0]
    s1 = probe_params["time"][1]
    dofx = probe_params["dofs"][0]
    dofy = probe_params["dofs"][1]

    assert_allclose(
        resp_common.yout[s0:s1, dofx], resp_pseudo_modal.yout[s0:s1, dofx], rtol=0.1
    )
    assert_allclose(
        resp_common.yout[s0:s1, dofy], resp_pseudo_modal.yout[s0:s1, dofy], rtol=0.1
    )

    assert_allclose(
        resp_pseudo_modal.yout[s0:s1, dofx], resp_add_to_RHS.yout[s0:s1, dofx]
    )
    assert_allclose(
        resp_pseudo_modal.yout[s0:s1, dofy], resp_add_to_RHS.yout[s0:s1, dofy]
    )

    freq = 68.96552
    abs_max = 1.53523e-4

    change_sign = np.where(np.diff(np.sign(resp_common.yout[s0:s1, dofx])))[0]
    freq1 = 1 / (t[change_sign[2]] - t[change_sign[0]])
    abs_max1 = np.max(np.abs(resp_common.yout[s0:s1, dofx]))

    assert_allclose(freq, freq1, rtol=1e-3)
    assert_allclose(abs_max, abs_max1, rtol=1e-3)



"""Tests file.
Tests for:
    st_shaft_element.py
    st_disk_element.py
    st_bearing_seal_element.py
    st_point_mass.py
"""

import sys

import numpy as np
import pytest
from plotly import graph_objects as go

from ross.stochastic.st_bearing_seal_element import ST_BearingElement
from ross.stochastic.st_disk_element import ST_DiskElement
from ross.stochastic.st_materials import ST_Material
from ross.stochastic.st_point_mass import ST_PointMass
from ross.stochastic.st_shaft_element import ST_ShaftElement


@pytest.fixture
def rand_shaft():
    E = [209e9, 211e9]
    st_steel = ST_Material(name="Steel", rho=7810, E=E, G_s=81.2e9)
    elm = ST_ShaftElement(
        L=[1.0, 1.1],
        idl=[0.01, 0.02],
        odl=[0.1, 0.2],
        material=st_steel,
        is_random=["L", "idl", "odl", "material"],
    )
    return elm


@pytest.fixture
def rand_disk_from_inertia():
    elm = ST_DiskElement(
        n=1, m=[30, 40], Id=[0.2, 0.3], Ip=[0.5, 0.7], is_random=["m", "Id", "Ip"]
    )
    return elm


@pytest.fixture
def rand_disk_from_geometry():
    E = [209e9, 211e9]
    st_steel = ST_Material(name="Steel", rho=7810, E=E, G_s=81.2e9)
    elm = ST_DiskElement.from_geometry(
        n=1,
        material=st_steel,
        width=[0.07, 0.08],
        i_d=[0.05, 0.06],
        o_d=[0.30, 0.40],
        is_random=["material", "width", "i_d", "o_d"],
    )
    return elm


@pytest.fixture
def rand_bearing_constant_coefs():
    kxx = [1e6, 2e6]
    cxx = [1e3, 2e3]
    elm = ST_BearingElement(n=1, kxx=kxx, cxx=cxx, is_random=["kxx", "cxx"])

    return elm


@pytest.fixture
def rand_bearing_varying_coefs():
    kxx = [[1e6, 1.1e6], [2e6, 2.1e6]]
    kxy = [[1.5e6, 1.6e6], [2.5e6, 2.6e6]]
    kyx = [[1.5e6, 1.6e6], [2.5e6, 2.6e6]]
    kyy = [[3e6, 3.1e6], [4e6, 4.1e6]]
    cxx = [[1e3, 1.1e3], [2e3, 2.1e3]]
    cxy = [[1.5e3, 1.6e3], [2.5e3, 2.6e3]]
    cyx = [[1.5e3, 1.6e3], [2.5e3, 2.6e3]]
    cyy = [[3e3, 3.1e3], [4e3, 4.1e3]]
    frequency = np.array([500, 800])
    elm = ST_BearingElement(
        n=1,
        kxx=kxx,
        kxy=kxy,
        kyx=kyx,
        kyy=kyy,
        cxx=cxx,
        cxy=cxy,
        cyx=cyx,
        cyy=cyy,
        frequency=frequency,
        is_random=["kxx", "kxy", "kyx", "kyy", "cxx", "cxy", "cyx", "cyy"],
    )

    return elm


@pytest.fixture
def rand_point_mass():
    mx = [2.0, 2.5]
    my = [3.0, 3.5]
    elm = ST_PointMass(n=1, mx=mx, my=my, is_random=["mx", "my"])
    return elm


###############################################################################
# testing attributes and parameters
###############################################################################
def test_st_shaft_element(rand_shaft):
    elm = list(iter(rand_shaft))
    assert [sh.L for sh in elm] == [1.0, 1.1]
    assert [sh.idl for sh in elm] == [0.01, 0.02]
    assert [sh.odl for sh in elm] == [0.1, 0.2]
    assert [sh.idr for sh in elm] == [0.01, 0.02]
    assert [sh.odr for sh in elm] == [0.1, 0.2]
    assert [sh.material.E for sh in elm] == [209000000000.0, 211000000000.0]


def test_st_disk_element_from_inertia(rand_disk_from_inertia):
    elm = list(iter(rand_disk_from_inertia))
    assert [dk.n for dk in elm] == [1, 1]
    assert [dk.m for dk in elm] == [30, 40]
    assert [dk.Id for dk in elm] == [0.2, 0.3]
    assert [dk.Ip for dk in elm] == [0.5, 0.7]


def test_st_disk_element_from_geometry(rand_disk_from_geometry):
    elm = list(iter(rand_disk_from_geometry))
    assert [dk.n for dk in elm] == [1, 1]
    assert [dk.m for dk in elm] == [37.570502893821185, 76.74810321754951]

    try:
        assert [dk.Id for dk in elm] == [0.23254575853654735, 0.8256816771154702]
        assert [dk.Ip for dk in elm] == [0.43440893970980743, 1.5694987107988876]
    except:
        assert [dk.Id for dk in elm] == [0.2325457585365474, 0.8256816771154702]
        assert [dk.Ip for dk in elm] == [0.43440893970980754, 1.5694987107988876]


def test_st_bearing_element_constant_coef(rand_bearing_constant_coefs):
    elm = list(iter(rand_bearing_constant_coefs))
    assert [brg.n for brg in elm] == [1, 1]
    assert [brg.kxx for brg in elm] == [[1000000.0], [2000000.0]]
    assert [brg.kyy for brg in elm] == [[1000000.0], [2000000.0]]
    assert [brg.kxy for brg in elm] == [[0], [0]]
    assert [brg.kyx for brg in elm] == [[0], [0]]
    assert [brg.cxx for brg in elm] == [[1000.0], [2000.0]]
    assert [brg.cyy for brg in elm] == [[1000.0], [2000.0]]
    assert [brg.cxy for brg in elm] == [[0], [0]]
    assert [brg.cyx for brg in elm] == [[0], [0]]


def test_st_bearing_element_varying_coef(rand_bearing_varying_coefs):
    elm = list(iter(rand_bearing_varying_coefs))
    assert [brg.n for brg in elm] == [1, 1]
    assert [list(brg.kxx) for brg in elm] == [
        [1000000.0, 2000000.0],
        [1100000.0, 2100000.0],
    ]
    assert [list(brg.kyy) for brg in elm] == [
        [3000000.0, 4000000.0],
        [3100000.0, 4100000.0],
    ]
    assert [list(brg.kxy) for brg in elm] == [
        [1500000.0, 2500000.0],
        [1600000.0, 2600000.0],
    ]
    assert [list(brg.kyx) for brg in elm] == [
        [1500000.0, 2500000.0],
        [1600000.0, 2600000.0],
    ]
    assert [list(brg.cxx) for brg in elm] == [[1000.0, 2000.0], [1100.0, 2100.0]]
    assert [list(brg.cyy) for brg in elm] == [[3000.0, 4000.0], [3100.0, 4100.0]]
    assert [list(brg.cxy) for brg in elm] == [[1500.0, 2500.0], [1600.0, 2600.0]]
    assert [list(brg.cyx) for brg in elm] == [[1500.0, 2500.0], [1600.0, 2600.0]]


def test_st_point_mass(rand_point_mass):
    elm = list(iter(rand_point_mass))
    assert [pm.n for pm in elm] == [1, 1]
    assert [pm.mx for pm in elm] == [2.0, 2.5]
    assert [pm.my for pm in elm] == [3.0, 3.5]


###############################################################################
# testing error messages
###############################################################################
def test_st_bearing_error_messages(rand_bearing_constant_coefs):
    kxx = [1e6, 2e6]
    cxx = [1e3, 2e3]
    freq = [500, 1000]
    with pytest.raises(ValueError) as ex:
        ST_BearingElement(
            n=1, kxx=kxx, cxx=cxx, frequency=freq, is_random=["kxx", "cxx", "frequency"]
        )
    assert "frequency can not be a random variable" in str(ex.value)

    with pytest.raises(ValueError) as ex:
        rand_bearing_constant_coefs.plot_random_var(["kxy"])
    assert (
        "Random variable not in var_list. Select variables from ['kxx', 'cxx', 'kyy', 'cyy']"
        in str(ex.value)
    )

    with pytest.raises(KeyError) as ex:
        rand_bearing_constant_coefs["odd"] = [1, 2]
    assert "Object does not have parameter: odd." in str(ex.value)

    with pytest.raises(KeyError) as ex:
        rand_bearing_constant_coefs["odd"]
    assert "Object does not have parameter: odd." in str(ex.value)


def test_st_disk_error_messages(rand_disk_from_inertia):
    with pytest.raises(ValueError) as ex:
        rand_disk_from_inertia.plot_random_var(["n"])
    assert (
        "Random variable not in var_list. Select variables from ['m', 'Id', 'Ip']"
        in str(ex.value)
    )

    with pytest.raises(KeyError) as ex:
        rand_disk_from_inertia["odd"] = [1, 2]
    assert "Object does not have parameter: odd." in str(ex.value)

    with pytest.raises(KeyError) as ex:
        rand_disk_from_inertia["odd"]
    assert "Object does not have parameter: odd." in str(ex.value)


def test_st_shaft_error_messages(rand_shaft):
    with pytest.raises(ValueError) as ex:
        rand_shaft.plot_random_var(["n"])
    assert (
        "Random variable not in var_list. Select variables from ['L', 'idl', 'odl', 'idr', 'odr']"
        in str(ex.value)
    )

    with pytest.raises(KeyError) as ex:
        rand_shaft["odd"] = [1, 2]
    assert "Object does not have parameter: odd." in str(ex.value)

    with pytest.raises(KeyError) as ex:
        rand_shaft["odd"]
    assert "Object does not have parameter: odd." in str(ex.value)


def test_st_point_mass_error_messages(rand_point_mass):
    with pytest.raises(ValueError) as ex:
        rand_point_mass.plot_random_var(["n"])
    assert "Random variable not in var_list. Select variables from ['mx', 'my']" in str(
        ex.value
    )

    with pytest.raises(KeyError) as ex:
        rand_point_mass["odd"] = [1, 2]
    assert "Object does not have parameter: odd." in str(ex.value)

    with pytest.raises(KeyError) as ex:
        rand_point_mass["odd"]
    assert "Object does not have parameter: odd." in str(ex.value)


###############################################################################
# testing elements plot type
###############################################################################
def test_element_stats_plot(
    rand_shaft, rand_disk_from_inertia, rand_bearing_constant_coefs, rand_point_mass
):
    figure_type = type(go.Figure())
    fig = rand_shaft.plot_random_var(["L"])
    assert type(fig) == figure_type

    fig = rand_disk_from_inertia.plot_random_var(["Id"])
    assert type(fig) == figure_type

    fig = rand_bearing_constant_coefs.plot_random_var(["kxx"])
    assert type(fig) == figure_type

    fig = rand_point_mass.plot_random_var(["mx"])
    assert type(fig) == figure_type



import pytest
from numpy.testing import assert_allclose

from ross import Probe, Q_


@pytest.fixture
def probe():
    return Probe(10, Q_(45, "degree"), tag="V1")


def test_parameters(probe):
    assert_allclose(probe.node, 10)
    assert_allclose(probe.angle, 0.7853981633974483)
    assert probe.direction == "radial"


def test_info(probe):
    node, angle, direc = probe.info
    assert_allclose(node, 10)
    assert_allclose(angle, 0.7853981633974483)
    assert direc == "radial"


@pytest.fixture
def probe2():
    return Probe(10, direction="axial", tag="AX1")


def test_parameters_axial(probe2):
    assert_allclose(probe2.node, 10)
    assert probe2.angle is None
    assert probe2.direction == "axial"


def test_info_axial(probe2):
    node, angle, direc = probe2.info
    assert_allclose(node, 10)
    assert angle is None
    assert direc == "axial"



import numpy as np
from scipy.optimize import least_squares, root


def calculate_attitude_angle(eccentricity_ratio):
    """Calculates the attitude angle based on the eccentricity ratio.
    Suitable only for short bearings.
    Parameters
    ----------
    eccentricity_ratio: float
        The ratio between the journal displacement, called just eccentricity, and
        the radial clearance.
    Returns
    -------
    float
        Attitude angle
    Examples
    --------
    >>> from ross.fluid_flow.fluid_flow import fluid_flow_example
    >>> my_fluid_flow = fluid_flow_example()
    >>> calculate_attitude_angle(my_fluid_flow.eccentricity_ratio) # doctest: +ELLIPSIS
    0.93...
    """
    return np.arctan(
        (np.pi * (1 - eccentricity_ratio**2) ** (1 / 2)) / (4 * eccentricity_ratio)
    )


def internal_radius_function(gama, attitude_angle, radius_rotor, eccentricity):
    """This function calculates the x and y of the internal radius of the rotor,
    as well as its distance from the origin, given the distance in the theta-axis,
    the attitude angle, the radius of the rotor and the eccentricity.
    Parameters
    ----------
    gama: float
        Gama is the distance in the theta-axis. It should range from 0 to 2*np.pi.
    attitude_angle: float
        Attitude angle. Angle between the origin and the eccentricity (rad).
    radius_rotor: float
        The radius of the journal.
    eccentricity: float
        The journal displacement from the center of the stator.
    Returns
    -------
    radius_internal: float
        The size of the internal radius at that point.
    xri: float
        The position x of the returned internal radius.
    yri: float
        The position y of the returned internal radius.
    Examples
    --------
    >>> from ross.fluid_flow.fluid_flow import fluid_flow_example
    >>> my_fluid_flow = fluid_flow_example()
    >>> attitude_angle = my_fluid_flow.attitude_angle
    >>> radius_rotor = my_fluid_flow.radius_rotor
    >>> eccentricity = my_fluid_flow.eccentricity
    >>> radius_internal, xri, yri = internal_radius_function(0, attitude_angle, radius_rotor, eccentricity)
    >>> radius_internal # doctest: +ELLIPSIS
    0.2...
    """
    if (np.pi / 2 + attitude_angle) < gama < (3 * np.pi / 2 + attitude_angle):
        alpha = np.absolute(3 * np.pi / 2 - gama + attitude_angle)
    else:
        alpha = gama + np.pi / 2 - attitude_angle
    radius_internal = np.sqrt(
        radius_rotor**2 - (eccentricity * np.sin(alpha)) ** 2
    ) + eccentricity * np.cos(alpha)
    xri = radius_internal * np.cos(gama)
    yri = radius_internal * np.sin(gama)

    return radius_internal, xri, yri


def external_radius_function(
    gama,
    radius_stator,
    radius_rotor=None,
    shape="cylindrical",
    preload=None,
    displacement=None,
    max_depth=None,
):
    """This function returns the x and y of the radius of the stator, as well as its distance from the
    origin, given the distance in the theta-axis and the radius of the bearing.
    Parameters
    ----------
    gama: float
        Gama is the distance in the theta-axis. It should range from 0 to 2*np.pi.
    radius_stator : float
        The external radius of the bearing.
    radius_rotor : float
        The internal radius of the bearing.
    shape : str
        Determines the type of bearing geometry.
        'cylindrical': cylindrical bearing;
        'eliptical': eliptical bearing;
        'wear': journal bearing wear.
        The default is 'cylindrical'.
    preload : float
        The ellipticity ratio of the bearing if the shape is eliptical. Varies between 0 and 1.
        The default is 0.05.
    displacement : float
        Angular displacement of the bearing wear in relation to the vertical axis.
    max_depth: float
        The maximum wear depth.
    Returns
    -------
    radius_external: float
        The size of the external radius at that point.
    xre: float
        The position x of the returned external radius.
    yre: float
        The position y of the returned external radius.
    Examples
    --------
    >>> from ross.fluid_flow.fluid_flow import fluid_flow_example
    >>> my_fluid_flow = fluid_flow_example()
    >>> radius_stator = my_fluid_flow.radius_stator
    >>> radius_external, xre, yre = external_radius_function(0, radius_stator)
    >>> radius_external
    0.2002
    """
    if shape == "eliptical":
        cr = radius_stator - radius_rotor
        elip = preload * cr
        if 0 <= gama <= np.pi / 2:
            alpha = np.pi / 2 + gama
        elif np.pi / 2 < gama <= np.pi:
            alpha = 3 * np.pi / 2 - gama
        elif np.pi < gama <= 3 * np.pi / 2:
            alpha = gama - np.pi / 2
        else:
            alpha = 5 * np.pi / 2 - gama

        radius_external = elip * np.cos(alpha) + np.sqrt(
            ((radius_stator) ** 2) - (elip * np.sin(alpha)) ** 2
        )
        xre = radius_external * np.cos(gama)
        yre = radius_external * np.sin(gama)

    elif shape == "wear":
        if max_depth == 0:
            d_theta = 0
        else:
            cr = radius_stator - radius_rotor
            theta_s = np.pi / 2 + np.arccos(max_depth / cr - 1) + displacement
            theta_f_0 = np.pi / 2 - np.arccos(max_depth / cr - 1) + displacement
            theta_f = 2 * np.pi + theta_f_0

            if theta_f <= 2 * np.pi:
                if theta_s <= gama <= theta_f:
                    d_theta = max_depth - cr * (
                        1 + np.cos(gama - np.pi / 2 - displacement)
                    )
                else:
                    d_theta = 0
            else:
                if theta_s <= gama <= 2 * np.pi:
                    d_theta = max_depth - cr * (
                        1 + np.cos(gama - np.pi / 2 - displacement)
                    )
                elif 0 <= gama <= theta_f_0:
                    gama2 = gama + 2 * np.pi
                    d_theta = max_depth - cr * (
                        1 + np.cos(gama2 - np.pi / 2 - displacement)
                    )
                else:
                    d_theta = 0

        radius_external = radius_stator + d_theta
        xre = radius_external * np.cos(gama)
        yre = radius_external * np.sin(gama)

    else:
        radius_external = radius_stator
        xre = radius_external * np.cos(gama)
        yre = radius_external * np.sin(gama)

    return radius_external, xre, yre


def reynolds_number(density, characteristic_speed, radial_clearance, viscosity):
    """Returns the reynolds number based on the characteristic speed.
    This number denotes the ratio between fluid inertia (advection) forces and viscous-shear forces.
        Parameters
        ----------
        density: float
            Fluid density(Kg/m^3).
        characteristic_speed: float
            Characteristic fluid speeds.
        radial_clearance: float
            Difference between both stator and rotor radius, regardless of eccentricity.
        viscosity: float
            Viscosity (Pa.s).
        Returns
        -------
        float
            The reynolds number.
        Examples
        --------
        >>> from ross.fluid_flow.fluid_flow import fluid_flow_example
        >>> my_fluid_flow = fluid_flow_example()
        >>> density = my_fluid_flow.density
        >>> characteristic_speed = my_fluid_flow.characteristic_speed
        >>> radial_clearance = my_fluid_flow.radial_clearance
        >>> viscosity = my_fluid_flow.viscosity
        >>> reynolds_number(density, characteristic_speed, radial_clearance, viscosity) # doctest: +ELLIPSIS
        24.01...
    """
    return (density * characteristic_speed * radial_clearance) / viscosity


def modified_sommerfeld_number(
    radius_stator, omega, viscosity, length, load, radial_clearance
):
    """Returns the modified sommerfeld number.
    Parameters
    ----------
    radius_stator : float
        The external radius of the bearing.
    omega: float
        Rotation of the rotor (rad/s).
    viscosity: float
        Viscosity (Pa.s).
    length: float
        Length in the Z direction (m).
    load: float
        Load applied to the rotor (N).
    radial_clearance: float
        Difference between both stator and rotor radius, regardless of eccentricity.

    Returns
    -------
    float
        The modified sommerfeld number.
    Examples
    --------
    >>> from ross.fluid_flow.fluid_flow import fluid_flow_example
    >>> my_fluid_flow = fluid_flow_example()
    >>> radius_stator = my_fluid_flow.radius_stator
    >>> omega = my_fluid_flow.omega
    >>> viscosity = my_fluid_flow.viscosity
    >>> length = my_fluid_flow.length
    >>> load = my_fluid_flow.load
    >>> radial_clearance = my_fluid_flow.radial_clearance
    >>> modified_sommerfeld_number(radius_stator, omega, viscosity,
    ...                            length, load, radial_clearance) # doctest: +ELLIPSIS
    0.33...
    """
    return (radius_stator * 2 * omega * viscosity * (length**3)) / (
        8 * load * (radial_clearance**2)
    )


def sommerfeld_number(modified_s, radius_stator, length):
    """Returns the sommerfeld number, based on the modified sommerfeld number.
    Parameters
    ----------
    modified_s: float
        The modified sommerfeld number.
    radius_stator : float
        The external radius of the bearing.
    length: float
        Length in the Z direction (m).
    Returns
    -------
    float
        The sommerfeld number.
    Examples
    --------
    >>> from ross.fluid_flow.fluid_flow import fluid_flow_example
    >>> my_fluid_flow = fluid_flow_example()
    >>> radius_stator = my_fluid_flow.radius_stator
    >>> omega = my_fluid_flow.omega
    >>> viscosity = my_fluid_flow.viscosity
    >>> length = my_fluid_flow.length
    >>> load = my_fluid_flow.load
    >>> radial_clearance = my_fluid_flow.radial_clearance
    >>> modified_s = modified_sommerfeld_number(radius_stator, omega, viscosity,
    ...                            length, load, radial_clearance) # doctest: +ELLIPSIS
    >>> sommerfeld_number(modified_s, radius_stator, length) # doctest: +ELLIPSIS
    10.62...
    """
    return (modified_s / np.pi) * (radius_stator * 2 / length) ** 2


def calculate_eccentricity_ratio(modified_s):
    """Calculate the eccentricity ratio for a given load using the sommerfeld number.
    Suitable only for short bearings.
    Parameters
    ----------
    modified_s: float
        The modified sommerfeld number.
    Returns
    -------
    float
        The eccentricity ratio.
    Examples
    --------
    >>> modified_s = 6.329494061103412
    >>> calculate_eccentricity_ratio(modified_s) # doctest: +ELLIPSIS
    0.04999...
    """
    coefficients = [
        1,
        -4,
        (6 - (modified_s**2) * (16 - np.pi**2)),
        -(4 + (np.pi**2) * (modified_s**2)),
        1,
    ]
    roots = np.roots(coefficients)
    roots = np.sort(roots[np.isclose(roots.imag, 1e-16)].real)

    def f(x):
        """Fourth degree polynomial whose root is the square of the eccentricity ratio.
        Parameters
        ----------
        x: float
            Fourth degree polynomial coefficients.
        Returns
        -------
        float
            Polynomial value f at x."""
        c = coefficients
        return c[0] * x**4 + c[1] * x**3 + c[2] * x**2 + c[3] * x**1 + c[4] * x**0

    for i in roots:
        if 0 <= i <= 1:
            roots = root(f, i, tol=1e-10).x[0]
            return np.sqrt(roots)
    raise ValueError("Eccentricity ratio could not be calculated.")


def calculate_rotor_load(
    radius_stator, omega, viscosity, length, radial_clearance, eccentricity_ratio
):
    """Returns the load applied to the rotor, based on the eccentricity ratio.
    Suitable only for short bearings.
    Parameters
    ----------
    radius_stator : float
        The external radius of the bearing.
    omega: float
        Rotation of the rotor (rad/s).
    viscosity: float
        Viscosity (Pa.s).
    length: float
        Length in the Z direction (m).
    radial_clearance: float
        Difference between both stator and rotor radius, regardless of eccentricity.
    eccentricity_ratio: float
        The ratio between the journal displacement, called just eccentricity, and
        the radial clearance.
    Returns
    -------
    float
        Load applied to the rotor.
    Examples
    --------
    >>> from ross.fluid_flow.fluid_flow import fluid_flow_example
    >>> my_fluid_flow = fluid_flow_example()
    >>> radius_stator = my_fluid_flow.radius_stator
    >>> omega = my_fluid_flow.omega
    >>> viscosity = my_fluid_flow.viscosity
    >>> length = my_fluid_flow.length
    >>> radial_clearance = my_fluid_flow.radial_clearance
    >>> eccentricity_ratio = my_fluid_flow.eccentricity_ratio
    >>> calculate_rotor_load(radius_stator, omega, viscosity,
    ...                      length, radial_clearance, eccentricity_ratio) # doctest: +ELLIPSIS
    37.75...
    """
    return (
        (
            np.pi
            * radius_stator
            * 2
            * omega
            * viscosity
            * (length**3)
            * eccentricity_ratio
        )
        / (8 * (radial_clearance**2) * ((1 - eccentricity_ratio**2) ** 2))
    ) * (np.sqrt((16 / (np.pi**2) - 1) * eccentricity_ratio**2 + 1))


def move_rotor_center(fluid_flow_object, dx, dy):
    """For a given step on x or y axis,
    moves the rotor center and calculates new eccentricity, attitude angle,
    and rotor center.

    Parameters
    ----------
    fluid_flow_object: A FluidFlow object.
    dx: float
        The step on x axis.
    dy: float
        The step on y axis.
    Returns
    -------
    None
    --------
    >>> from ross.fluid_flow.fluid_flow import fluid_flow_example2
    >>> my_fluid_flow = fluid_flow_example2()
    >>> move_rotor_center(my_fluid_flow, 0, 0)
    >>> my_fluid_flow.eccentricity # doctest: +ELLIPSIS
    2.5...
    """
    fluid_flow_object.xi = fluid_flow_object.xi + dx
    fluid_flow_object.yi = fluid_flow_object.yi + dy
    fluid_flow_object.eccentricity = np.sqrt(
        fluid_flow_object.xi**2 + fluid_flow_object.yi**2
    )
    fluid_flow_object.eccentricity_ratio = (
        fluid_flow_object.eccentricity / fluid_flow_object.radial_clearance
    )
    fluid_flow_object.attitude_angle = np.arccos(
        abs(fluid_flow_object.yi / fluid_flow_object.eccentricity)
    )


def move_rotor_center_abs(fluid_flow_object, x, y):
    """Moves the rotor center to the coordinates (x, y) and calculates new eccentricity,
    attitude angle, and rotor center.
    Parameters
    ----------
    fluid_flow_object: A FluidFlow object.
    x: float
        Coordinate along x axis.
    y: float
        Coordinate along y axis.
    Returns
    -------
    None
    --------
    >>> from ross.fluid_flow.fluid_flow import fluid_flow_example
    >>> my_fluid_flow = fluid_flow_example()
    >>> move_rotor_center_abs(my_fluid_flow, 0, -1e-3*my_fluid_flow.radial_clearance)
    >>> my_fluid_flow.eccentricity # doctest: +ELLIPSIS
    1.99...
    """
    fluid_flow_object.xi = x
    fluid_flow_object.yi = y
    fluid_flow_object.eccentricity = np.sqrt(
        fluid_flow_object.xi**2 + fluid_flow_object.yi**2
    )
    fluid_flow_object.eccentricity_ratio = (
        fluid_flow_object.eccentricity / fluid_flow_object.radial_clearance
    )
    fluid_flow_object.attitude_angle = np.arccos(
        abs(fluid_flow_object.yi / fluid_flow_object.eccentricity)
    )



import sys

import pytest
from numpy.testing import assert_allclose

from ross.materials import *


@pytest.fixture
def AISI4140():
    return Material(
        name="AISI4140", rho=7850, E=203200000000.0, G_s=80000000000.0, color="#525252"
    )


@pytest.fixture
def A216WCB():
    return Material(
        name="A216WCB",
        rho=7820.0,
        E=210000000000.0,
        G_s=81395348837.2093,
        color="#525252",
    )


def test_material_equality(A216WCB, AISI4140):
    assert A216WCB != AISI4140


def test_raise_name_material():
    with pytest.raises(ValueError) as excinfo:
        Material("with space", rho=7850, G_s=80e9, Poisson=0.27)
    assert "Spaces are not allowed" in str(excinfo.value)


def test_E():
    mat = Material(name="test", rho=7850, G_s=80e9, Poisson=0.27)
    assert_allclose(mat.E, 203.2e9)
    assert_allclose(mat.G_s, 80e9)
    assert_allclose(mat.Poisson, 0.27)


def test_G_s():
    mat = Material(name="test", rho=7850, E=203.2e9, Poisson=0.27)
    assert_allclose(mat.E, 203.2e9)
    assert_allclose(mat.G_s, 80e9)
    assert_allclose(mat.Poisson, 0.27)


def test_Poisson():
    mat = Material(name="test", rho=7850, E=203.2e9, G_s=80e9)
    assert_allclose(mat.E, 203.2e9)
    assert_allclose(mat.G_s, 80e9)
    assert_allclose(mat.Poisson, 0.27)


def test_E_G_s_Poisson():
    mat = Material(name="test", rho=7850, E=203.2e9, G_s=80e9)
    assert_allclose(mat.E, 203.2e9)
    assert_allclose(mat.G_s, 80e9)
    assert_allclose(mat.Poisson, 0.27)


def test_specific_material(AISI4140):
    assert_allclose(AISI4140.rho, 7850)
    assert_allclose(AISI4140.E, 203.2e9)
    assert_allclose(AISI4140.G_s, 80e9)
    assert_allclose(AISI4140.Poisson, 0.27)


def test_error_rho():
    with pytest.raises(TypeError) as ex:
        Material(name="test", E=203.2e9, G_s=80e9)
    assert "__init__() missing 1 required positional argument: 'rho'" in str(ex.value)


def test_error_E_G_s_Poisson():
    with pytest.raises(ValueError) as ex:
        Material(name="test", rho=785, E=203.2e9)
    assert "Exactly 2 arguments from E" in str(ex.value)
    with pytest.raises(ValueError) as ex:
        Material(name="test", rho=785, E=203.2e9, G_s=80e9, Poisson=0.27)
    assert "Exactly 2 arguments from E" in str(ex.value)


# Serialization tests.


def test_available_materials():
    Material.available_materials()


def test_serialization():
    available = Material.available_materials()
    obj1 = Material(name="obj1", rho=92e1, E=281.21, G_s=20e9)
    obj1.save_material()
    obj2 = Material.load_material("obj1")
    assert obj1.__dict__ == obj2.__dict__

    obj1.remove_material("obj1")
    available_after = Material.available_materials()
    assert available == available_after


def test_repr():
    mat0 = Material(name="obj1", rho=92e1, E=281.21, G_s=20e9)
    mat1 = eval(repr(mat0))
    assert mat0 == mat1
